#Written by Emma Tait for FEMC Forest Health Atlas
#This script cleans and recodes the IDS yearly Aerial Survey Data
# for VT ME MA NY NH RI CT for all data from 2017 onwards. 
#Last Updated 10 September 2019

## ****** IMPORTANT NOTE ********
# FEMC updates and standardizes the VT data separately to also update the VT Sketchmapping project
# and then merges the VT data in after it has been processed. 


import arcpy
import os

#********* Variables Required. Change as necessary *********

#current year being added to regional file
year = '2017'

#path where the outputs of the script will be saved
workingDirectory = 'I:\\WorkingProjects\\ForestHealthAtlas\\RegionalProcessingWork\\Individual_Yearly_regional_FNF\\'

# file (including path) with raw IDS data for the year being added
rawIDS = r'I:\WorkingProjects\ForestHealthAtlas\USFSDelivery_ADS\IDS2_NA_2017\IDS_NA_2017.gdb\DAMAGE_AREAS_FLAT_NA_2017'

#file of state boundaries to clip features to. Must include an attribute field containing capitalized state abbreviations as a string. Possible states include VT, NY, NH, ME, MA, CT, RI
state_boundaries = 'I:\\WorkingProjects\\ForestHealthAtlas\\RegionalProcessingWork\\regional_state_boundaries_noVT_reproj.shp'



#************************************
#set working environment
arcpy.env.overwriteOutput = True;
arcpy.env.workspace = workingDirectory

def clipRegion_deleteFields(year, path, rawIDS, state_boundaries):   
    #create workspace, allow overwriting outputs
    

    #get the fields from the IDS dataset
    rawIDS_fields = arcpy.ListFields (rawIDS)
    #get the state abbreviations field
    stateField = arcpy.ListFields(state_boundaries,'STATE_ABBR')
    #create variable for list of new fields
    newFieldList =[]
    #add all field names  from the IDS data to new list
    for f in rawIDS_fields:
        newFieldList.append(f.name)
    #add state abbreviations field to new list
    newFieldList.append(stateField[0].name)

    #create field mappings
    fieldmappings = arcpy.FieldMappings()
    #add both datasets
    fieldmappings.addTable(rawIDS)
    fieldmappings.addTable(state_boundaries)

    # through field mappings and keep only those that are in the new fields list
    for f in fieldmappings:
         n = (f.outputField).name 
         try:
             newFieldList.index(n)
         except ValueError:
             fieldmappings.removeFieldMap(fieldmappings.findFieldMapIndex(n))
    #rename the state abbreviations field to match ADS standards
    stateIndex = fieldmappings.findFieldMapIndex('STATE_ABBR')
    fieldmap = fieldmappings.getFieldMap(stateIndex)
    field = fieldmap.outputField
    field.name = 'STATE'
    fieldmap.outputField = field
    fieldmappings.replaceFieldMap(stateIndex,fieldmap)


    #make workspace output geodatabase and see if it will write
    #or create file first, programmatically and then write rows to it?

    arcpy.env.overwriteoutput = True;
    #create spatial join to get only the features inside the state boundaries, populate state abbrevation for each feature according to spatial join
    clipfile = arcpy.SpatialJoin_analysis(rawIDS,state_boundaries, 'regional_IDS_'+year+'.shp','','KEEP_COMMON',fieldmappings,'COMPLETELY_WITHIN')
    def unique_values(table , field1,field2):
        with arcpy.da.SearchCursor(table, [field1,field2]) as cursor:
            return sorted({row for row in cursor})
    print 'Code: group'
    #print a list of the host group codes and host group names
    codes = unique_values(clipfile,'HOST_GROUP','HOST_GRO_1')
    for c in codes:
        if c[0] != None:
            print str(c[0])+': '+str(c[1])
    print 'Clip Region and Delete Extra Fields script complete'
    return clipfile

def IDS_RecodetoOldStandard(year, path, regionalFile):
    finalFields=["OBJECTID","SHAPE","Shape","FID","DATE","DCA1","DCA2","DCA3","DMG","DMG_TYPE1","DMG_TYPE2","DMG_TYPE3","DMGNAME","DMGPATT","DMGSEV","DMGTYP","DMGTYPE","FOR_TYPE1","FOR_TYPE2","FOR_TYPE3","HOST1","HOST2","HOST3","HOST4","HOST5","NO_TREES1","NO_TREES2","NO_TREES3","NOTES","PATTERN1","PATTERN2","PATTERN3","PCT_MORT1","PCT_MORT2","PCT_MORT3","RPT_YR","SEVERITY1","SEVERITY2","SEVERITY3","Shape_Area","Shape_Length","STATE","SURVEY_ID1","SURVEY_ID2","SURVEY_ID3","SURVEYOR","TPA1","TPA2","TPA3","VTDMGAGT","VTDMGHOST","VTDMGSEV","VTDMGTYP","VTHOST"]
    year = int(year)

    arcpy.env.workspace = path
    files=arcpy.ListFeatureClasses("*")
    print regionalFile
    merges=[]



    #Iterate through the resulting dataset to update codes

    ##if False:
    arcpy.AddField_management(regionalFile,'DCA1','LONG',10)
    arcpy.AddField_management(regionalFile,'DCA2','LONG',10)
    arcpy.AddField_management(regionalFile,'DCA3','LONG',10)
    arcpy.AddField_management(regionalFile,'DMG_TYPE1','LONG',5)
    arcpy.AddField_management(regionalFile,'HOST1','LONG',5)
    arcpy.AddField_management(regionalFile,'HOST2','LONG',5)
    arcpy.AddField_management(regionalFile,'HOST3','LONG',5)
    arcpy.AddField_management(regionalFile,'HOST4','LONG',5)
    arcpy.AddField_management(regionalFile,'HOST5','LONG',5)
    arcpy.AddField_management(regionalFile,'FOR_TYPE1','LONG',5)
    arcpy.AddField_management(regionalFile,'STATE','TEXT',50)
    arcpy.AddField_management(regionalFile,'RPT_YR','SHORT')
    arcpy.AddField_management(regionalFile,'SEVERITY1','LONG',10)
    rows = arcpy.UpdateCursor(regionalFile, "", "", "DCA1,DCA2,DCA3,HOST_GROUP,DAMAGE_TYP,DCA_CODE,DMG_TYPE1,HOST1,HOST2,HOST3,HOST4,HOST5,FOR_TYPE1,SEVERITY1,STATE,RPT_YR")
    ##"DCA1,DCA2,DCA3,DMG_TYPE1,DMG_TYPE2, DMG_TYPE3, HOST1,HOST2,HOST3,HOST4,HOST5,PATTERN1, PATTERN2,PATTERN3,FOR_TYPE1,FOR_TYPE2,FOR_TYPE3,SEVERITY1,SEVERITY2,SEVERITY3,STATE"
    for row in rows:
        #add year
        row.RPT_YR = year
        #severity is not calculated
        row.SEVERITY1 = -1
        #Put damage agent and type in appropriate columns
        row.DCA1=row.DCA_CODE
        row.DCA2=99999
        row.DCA3=99999
        if row.DAMAGE_TYP==12 or row.DAMAGE_TYP==13 or row.DAMAGE_TYP==14:
                row.DMG_TYPE1=1
        elif row.DAMAGE_TYP==18 or row.DAMAGE_TYP==19:
                row.DMG_TYPE1=10
        else:
                row.DMG_TYPE1=row.DAMAGE_TYP
        #HOST group is red and white pine
        if row.HOST_GROUP ==9757:
            row.HOST1 = 129 #white pine
            row.HOST2 = 125 #red pine
            row.HOST3=-1
            row.FOR_TYPE1 = 000 #White/Red/Jack pine group? or 9000 mixed conifers?
        #HOST Group is red, black and white oak, yellow birch, red maple
        if row.HOST_GROUP == 9755:
            row.HOST1 = 833 #n. red oak
            row.HOST1 = 802 #black oak
            row.HOST3 = 837 #white oak
            row.HOST4 = 371 #yellow birch
            row.HOST2 = 316 #red maple
            row.FOR_TYPE1 = 890 #mixed northern hardwood
        #HOST group is red, jack, scotch and white pine
        elif row.HOST_GROUP == 9754: 
            row.HOST1 = 125 #red pine
            row.HOST2 = 105 #jack pine
            row.HOST3 = 129 #white pine
            row.FOR_TYPE1 = 000 #White/Red/Jack pine group? or 9000 mixed conifers?
        #HOST Group is white and green ash
        elif row.HOST_GROUP == 9751:
            row.HOST1 = 541 #white ash
            row.HOST2 = 544 #green ash
            row.HOST3=-1
            row.FOR_TYPE1 = 6524 #ash? 890 mixed northern hardwoods?
        #HOST group is white and northern red oak, red maple
        elif row.HOST_GROUP == 9750:
            row.HOST1 = 802 #white oak
            row.HOST2 = 833 #n. red oak
            row.HOST3 = 316 #red maple
            row.FOR_TYPE1 = 890 #mixed northern hardwoods
        #HOST Group is blue and norway spruce, white pine
        elif row.HOST_GROUP == 9749:
            row.HOST1 = 96 #blue spruce
            row.HOST2 = 91 #norway spruce
            row.HOST3 = 129 #white pine
            row.FOR_TYPE1 = 100 #spruce/fir group? or 9000 mixed conifers
        #HOST Group 9747 is chestnut, white and northern red oak  
        elif row.HOST_GROUP == 9747: 
            row.HOST1 = 832 #Chestnut oak
            row.HOST2 = 802 #white oak
            row.HOST3 = 833 #n. red oak
            row.FOR_TYPE1 = 530 #white oak/red oak/hickory or 590 mixed central hardwoods 0r 6800 oak (decidious)?
        #HOST group 9744 is chestnut, white, scarlet, black and n. red oak
        elif row.HOST_GROUP == 9744:
            row.HOST1 = 832 #Chestnut oak
            row.HOST2 = 802 #white oak
            row.HOST3 = 806 #scarlet oak
            row.HOST4 = 837 #black oak
            row.HOST5 = 833 #n. red oak
            row.FOR_TYPE1 =  590 #mixed central hardwoods 0r 6800 oak (decidious)?
        #HOST Group is balsam fir and red spruce
        elif row.HOST_GROUP == 9741:
            row.HOST1 = 12 #balsim fir
            row.HOST2 = 97 #red spruce
            row.HOST3=-1
            row.FOR_TYPE1 = 130 #red spruce/ balsam fir
        #HOST Group white, black and green ash
        elif row.HOST_GROUP == 9740 or row.HOST_GROUP == 9711:
            row.HOST1 = 541 #white ash
            row.HOST2 = 543 #black ash
            row.HOST3 = 544 #green ash
            row.FOR_TYPE1 = 6542 #ash?
        #HOST group is black and northern oak
        elif row.HOST_GROUP == 9739:
            row.HOST1 = 837 #black oak
            row.HOST2 = 833 #n. red oak
            row.HOST3=-1
            row.FOR_TYPE1 = 6800 #oak (decidious)? 890 mixed northern hardwoods?
        #HOST group is red, white, black and norway spruce
        elif row.HOST_GROUP == 9738:
            row.HOST1 = 97 #red spruce
            row.HOST2 = 94 #white spruce
            row.HOST3 = 91 #norway spruce
            row.FOR_TYPE1 = 100 #spruce fir group or 9001 mixed conifers
        #HOST group is white, red and pitch pine
        elif row.HOST_GROUP == 9737:
            row.HOST1 = 129 #white pine
            row.HOST2 = 125 #red pine
            row.HOST3 = 126 #pitch pine
            row.FOR_TYPE1 = 000 #white/red/jack pine group or 9000 mixed conifers
        #HOST group is white and northern red oak
        elif row.HOST_GROUP == 9736:
            row.HOST1 = 802 #white oak
            row.HOST2 = 833 #n. red oak
            row.HOST3=-1
            row.FOR_TYPE1 = 6800 #oak (decidious)? 890 mixed northern hardwoods?
        #HOST group is sugar and red maple, beech, yellow birch
        elif row.HOST_GROUP == 9735:
            row.HOST1=318 #Sugar maple
            row.HOST2=316 #Red Maple	
            row.HOST3=371 #Yellow birch
            row.FOR_TYPE1=810 #Sugar maple/beech/yellow birch
        #HOST group is sugar and red maple
        elif row.HOST_GROUP == 9734:
            row.HOST1=318 #Sugar maple
            row.HOST2=316 #Red Maple
            row.HOST3=-1
            row.FOR_TYPE1=890 #Mixed northern hardwoods looks most appropriate
        #HOST group is paper, yellow and grey birch
        elif row.HOST_GROUP == 9733:
            row.HOST1 = 375 #paper birch
            row.HOST2 = 371 #yellow birch
            row.HOST3 = 379 #gray birch
            row.FOR_TYPE1 = 6370 #birch? 890 mixed northern hardwoods?
        #HOST group is sugar and red maple, black and northern red oak
        elif row.HOST_GROUP == 9732:
            row.HOST1 = 318 #sugar maple
            row.HOST2 = 316 #red maple
            row.HOST3 = 837 #black oak
            row.HOST4 = 833 #red oak
            row.FOR_TYPE1=890 #Mixed northern hardwoods 
        #HOST group is white, jack, red, scotch and austrian pine
        elif row.HOST_GROUP == 9729:
            row.HOST1 = 129 #white pine
            row.HOST2 = 105 #jack pine
            row.HOST3 = 125 #red pine
            row.HOST4 = 130 #scotch
            row.HOST5 = 136 #austrian pine
            row.FOR_TYPE1 = 9000 #mixed conifers? 000 white/red/jack pine group
        #HOST group is sugar maple, yellow and paper birch, american beech
        elif row.HOST_GROUP == 9725:
            row.HOST1 = 318 #sugar maple
            row.HOST2 = 371 #yellow birch
            row.HOST3 = 375 #paper birch
            row.Host4 = 531 #american beech
            row.FOR_TYPE1 = 890 #Mixed northern hardwoods
        #HOST group 9720 is white, black, bur, northern red oak and northern pin oak and 
        elif row.HOST_GROUP == 9720:
            row.HOST1 = 802 #white oak
            row.HOST2 = 837 #black oak
            row.HOST3 = 823#bur oak
            row.HOST4 = 833 #n. red oak
            row.HOST5 = 809 #n pin oak
            row.FOR_TYPE1 = 6800 #oak (decidious)? 890 mixed northern hardwoods?
            #HOST group 9709 is white, bur, n.red oak, n. pin oak
        elif row.HOST_GROUP == 9709:
            row.HOST1 = 802 #white oak
            row.HOST2 = 823 #bur oak
            row.HOST3 = 833 #n. red oak
            row.HOST4 = 809 #n pin oak
            row.FOR_TYPE1 = 6800 #oak (decidious)? 890 mixed northern hardwoods?
        #HOST GROUP IS AMERICAN ELM, SUGAR AND RED MAPLE, WHITE ASH
        elif row.HOST_GROUP  == 9712:
            row.HOST1 = 972 #american elm
            row.HOST2 = 318 #sugar maple
            row.HOST3 = 316 #red maple 
            row.HOST4 = 541 #white ash
            row.FOR_TYPE1 = 890 #Mixed northern hardwoods
        #HOST group is sugar maple, black and red oak
        elif row.HOST_GROUP == 9706:
            row.HOST1 = 318 #sugar maple
            row.HOST2 = 873 #black oak
            row.HOST3 = 833 #n. red oak
            row.FOR_TYPE1 = 890 #mixed northern hardwoods 
        #HOST group is balsam fir, white and black spruce,
        elif row.HOST_GROUP == 9705:
            row.HOST1 = 12 #balsam fir
            row.HOST2 = 94 #white spruce
            row.HOST3 = 95 #black spruce
            row.FOR_TYPE1 = 9001 #mixed morthern hardwoods? or 100 Spruce/fir group
        #HOST group is northern red oak, paper birch
        elif row.HOST_GROUP == 9702:
            row.HOST1 = 833 #n. red oak
            row.HOST2 = 375 #paper birch
            row.FOR_TYPE1 = 890 #mixed northern hardwoods
        #HOST group is black and northern red oak, red maple
        elif row.HOST_GROUP == 9701:
            row.HOST1 = 937 #black oak
            row.HOST2 = 833 #n. red oak
            row.HOST3 = 316 #red maple
            row.FOR_TYPE1 = 890 #mixed northern hardwoods?
        else:
            if row.HOST_CODE==9998: #They added this code in 2016 for all tree species, just call it unknown
                row.HOST1=999
            elif row.HOST_CODE == None or row.HOST_CODE == 0:
                row.HOST1 = -1
            else:
                row.HOST1=row.HOST_CODE
                row.HOST2=-1
                row.HOST3=-1
                row.FOR_TYPE1=-1
        if row.HOST3 == None or row.HOST3 == 0:
            row.HOST3 = -1
        if row.HOST4 == None or row.HOST4 == 0:
            row.HOST4 = -1
        if row.HOST5 == None or row.HOST5 == 0:
            row.HOST5 = -1
        rows.updateRow(row)

    #Loop through files and delete fields that are not in the final list
    field_list = arcpy.ListFields(regionalFile)
    for field in field_list:
        if field.name not in finalFields and "OBJECTID" not in field.name and "SHAPE_Length" not in field.name and "SHAPE_Area" not in field.name:
##            print "Deleting field " + field.name + " in file "+regionalFile
            arcpy.DeleteField_management(regionalFile,  field.name )


    #Add area calculations
    arcpy.AddField_management(regionalFile,'Area_Ac','Long',10)
    arcpy.AddField_management(regionalFile,'Area_Ha','Long',10)
    arcpy.CalculateField_management(regionalFile, "Area_Ac","!SHAPE.AREA@ACRES!","PYTHON_9.3")
    arcpy.CalculateField_management(regionalFile, "Area_Ha","!SHAPE.AREA@HECTARES!","PYTHON_9.3")

    print 'IDS Standardization script complete' 
    return regionalFile

def mergeRegional (path, region):
    regionYearlyDataset = 'regionalIDS_'+year+'_CleanedStandardized.shp'
    finalFields=["Shape","FID","DCA1","DCA2","DCA3","DMG_TYPE1","DMG_TYPE2","DMG_TYPE3","FOR_TYPE1","FOR_TYPE2","FOR_TYPE3","HOST1","HOST2","HOST3","HOST4","HOST5","NO_TREES1","NO_TREES2","NO_TREES3","PCT_MORT1","PCT_MORT2","PCT_MORT3","RPT_YR","SEVERITY1","SEVERITY2","SEVERITY3","Shape_Area","Shape_Length","STATE",'Area_Ac','Area_Ha']
    fields = arcpy.ListFields(region)
    regionYearly = arcpy.Copy_management(region, path+regionYearlyDataset)
    field_list = arcpy.ListFields(regionYearly)
    for field in field_list:
        if field.name not in finalFields and "OBJECTID" not in field.name:
            arcpy.DeleteField_management(regionYearly,  field.name)   
    print 'yearly dataset made'
    return regionYearly



#**********Run Functions*********

clipfile = clipRegion_deleteFields(year,workingDirectory, rawIDS, state_boundaries)

standardRegion = IDS_RecodetoOldStandard(year, workingDirectory, clipfile)

regional = mergeRegional(workingDirectory, standardRegion)
print 'Script Complete'
